#!/usr/bin/env python

"""Some useful functions for number theory, e.g.:
	>>> for i in xrange(10):
	...     if i % 2 == 0 and is_prime(i):
	...         print i
	...
	2
	>>>
"""

def divisors(N):
    """divisors(N) -> tuple
Returns all the divisors of the positive integer N
Examples:
	>>> divisors(6)
	(1, 2, 3)
	>>> divisors(2009)
	(1, 7, 41, 49, 287)
	>>>
"""

    l = []
    for i in range(1, N):
        if N % i == 0:
            l.append(i)
    return tuple(l)

def is_prime(N):
    """is_prime(N) -> bool
Tests if the positive integer N is prime
Examples:
	>>> is_prime(2)
	True
	>>> is_prime(9)
	False
	>>> is_prime(2009)
	False
	>>>
"""
    if N < 2:
        return False
    for i in range(2, N):
        if N % i == 0:
            return False
    return True

def prime_divisors(N):
    """prime_divisors(N) -> tuple
Returns all the prime divisors of the positive integer N
Examples:
	>>> prime_divisors(2009)
	(7, 41)
	>>>
"""
    l = []
    for i in divisors(N):
        if is_prime(i):
            l.append(i)
    return tuple(l)

def twin_primes(N):
    """twin_primes(N) -> tuple
Returns all the twin primes (p1, p2) with p1, p2 <= N
Examples:
	>>> twin_primes(100)
	((3, 5), (5, 7), (11, 13), (17, 19), (29, 31), (41, 43), (59, 61), (71, 73))
	>>>
"""
    l = []
    p = 2
    for i in range(3, N + 1):
        if is_prime(i):
            if i - p == 2:
                l.append((p, i))
            p = i
    return tuple(l)
            
##############
if __name__ == "__main__":
    import doctest
    doctest.testmod()
    
